/*

To test manually, run this 3 commands:

- Terminal A: `nc -l -u -p 9503`
- Terminal B: `cargo run -p udp_over_tcp`
- Terminal C: `nc -p 9500 -u 127.0.0.1 9501`

Terminals A and C should be connected through the UDP-over-TCP connection

*/

use tokio::{
	runtime,
	spawn,
};

mod client;
mod loops;
mod server;

// The ephemeral UDP port that the PTTH_QUIC client will bind
const PORT_0: u16 = 9500;

// The well-known UDP port that the UDP-over-TCP client will bind
// The PTTH_QUIC client must connect to this instead of the real relay address
const PORT_1: u16 = 9501;

// The well-known TCP port that the UDP-over-TCP server will bind
const PORT_2: u16 = 9502;

// The well-known UDP port that the PTTH_QUIC relay will bind
const PORT_3: u16 = 9503;

fn main () -> anyhow::Result <()> {
	let rt = runtime::Runtime::new ()?;
	
	rt.block_on (async_main ())?;
	
	Ok (())
}

async fn async_main () -> anyhow::Result <()> {
	tokio::select! {
		_val = spawn (client::main ()) => {
			println! ("Client exited, exiting");
		},
		_val = spawn (server::main ()) => {
			println! ("Server exited, exiting");
		},
	}
	
	Ok (())
}
